unit DiskEjectComp;
{------------------------------------------------------------------
            TDiskEject Component for Delphi
------------------------------------------------------------------
(c) 1998 Daniel Polistchuck
ZIP AND JAZ Disks & Drives are copyrighted by IOMEGA
------------------------------------------------------------------
This source code is freeware, feel free to use it as
you wish - either commercially or not. The author shall
not be held responsible for any damage made by the use
or misuse of this software material.

Please, if any changes or corrections are made, send
me a copy at danpol@pobox.com.
------------------------------------------------------------------
Observations:
-Ejects Removable disks (ZIP,JAZ,etc.)
-Um abrao para a galera Delphizera do Brasil!
 (this is in portuguese and is a salute to the Delphi programmers
  from Brazil).
-Not tested under NT. Should work, though.
-While manipulating in design-time the Ejected property in
 the Object Inspector will only reflect the true state of
 the disk if you de-select then reselect the property after
 changing it from false to true. THIS PROBLEM OCCURS ONLY IN
 DESIGN-TIME, TOUGH.
------------------------------------------------------------------
Version history:
v1.0 - first public release

}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;


type
  EDiskEject = class (Exception);
  TDiskEject = class(TComponent)
  private
    { Private declarations }
    FDrive : Char;
    function GetEjected : Boolean;
    procedure SetEjected (Value : Boolean);
  protected
    { Protected declarations }
  public
    { Public declarations }
    procedure Eject;
  published
    { Published declarations }
    property Drive : Char read FDrive write FDrive;
    property Ejected : Boolean read GetEjected write SetEjected stored false;
  end;

procedure Register;

implementation

uses
  DiskEjectIO;

function IsDiskInDrive (Drive : Char) : Boolean;
var
  a,b,c,d : Integer;
begin
  Result := GetDiskFreeSpace (PChar(Drive+':\'),a,b,c,d);
end;

function EjectRemovableDrive (Drive : Char) : Boolean;
var
  reg : DEVIOCTL_REGISTERS;
  hDevice : THandle;
  cb : DWORD;
begin
  if Win32Platform = VER_PLATFORM_WIN32_WINDOWS then
  begin
    reg.reg_EAX := $440D;       //* IOCTL for block devices */
    reg.reg_EBX := Ord(UpCase(Drive))-Ord ('A')+1;       //* zero-based drive ID     */
    reg.reg_ECX := MakeWOrd($49,8);

    if (not Win95IOCTL(@reg)) then
    begin
      result := FALSE;
      exit;
    end;

    if (reg.reg_Flags and $0001) <> 0 then //* error if carry flag set */
    begin
      result := FALSE;
      exit;
    end;
    Result := True;
  end
  else
    if Win32Platform = VER_PLATFORM_WIN32_NT then
    begin
      hDevice := CreateFile(PChar('\\.\'+Drive+':'),
          0, FILE_SHARE_READ or FILE_SHARE_WRITE,
          nil, OPEN_EXISTING, 0, 0);

      if (hDevice = 0) then
      begin
        Result := False;
        Exit;
      end;

      Result := DeviceIoControl(hDevice,
          IOCTL_DISK_EJECT_MEDIA , nil, 0,
          nil,0, cb, nil);

      CloseHandle(hDevice);
    end
    else
      raise EDiskEject.Create ('Operating System not supported!');
end;

procedure TDiskEject.Eject;
begin
  if not EjectRemovableDrive (FDrive) then
    raise EDiskEject.Create ('Could not eject disk!');
end;

function TDiskEject.GetEjected : Boolean;
begin
  Result := not IsDiskInDrive (FDrive);
end;

procedure TDiskEject.SetEjected(Value : Boolean);
begin
  if Value then
    if not Ejected then
      Eject
    else
      raise EDiskEject.Create ('No disk in drive!');
end;

procedure Register;
begin
  RegisterComponents('Standard', [TDiskEject]);
end;

end.
